/*
//
//       Elvees Performance Platform
//           vector processing
//
*/

#pragma once

#include "eppdefs.h"
#include <math.h>   // for inline functions

#ifdef __cplusplus
extern "C" {
#endif


/*
//  Name:       eppv_GetLibraryVersion
//  Purpose:    gets the version of the library
//  Returns:    structure containing information about the current version of the EPPV library
//  Parameters: None
*/
EPPAPI(const epp_LibraryVersion*, eppv_GetLibraryVersion, (void))


/*
//  Name:        eppv_Copy
//  Purpose:     Copies vector data
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Copy_u8, (const epp_u8* pSrc, epp_u8* pDst, epp_size length))
EPPINL(epp_Status, eppv_Copy_s8, (const epp_s8* pSrc, epp_s8* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length);
}
EPPINL(epp_Status, eppv_Copy_u16, (const epp_u16* pSrc, epp_u16* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_u16));
}
EPPINL(epp_Status, eppv_Copy_s16, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s16));
}
EPPINL(epp_Status, eppv_Copy_u32, (const epp_u32* pSrc, epp_u32* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_u32));
}
EPPINL(epp_Status, eppv_Copy_s32, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s32));
}
EPPINL(epp_Status, eppv_Copy_u64, (const epp_u64* pSrc, epp_u64* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_u64));
}
EPPINL(epp_Status, eppv_Copy_s64, (const epp_s64* pSrc, epp_s64* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s64));
}
EPPINL(epp_Status, eppv_Copy_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_f16));
}
EPPINL(epp_Status, eppv_Copy_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_f32));
}
EPPINL(epp_Status, eppv_Copy_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_f64));
}
EPPINL(epp_Status, eppv_Copy_cs16, (const epp_cs16* pSrc, epp_cs16* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cs16));
}
EPPINL(epp_Status, eppv_Copy_cs32, (const epp_cs32* pSrc, epp_cs32* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cs32));
}
EPPINL(epp_Status, eppv_Copy_cs64, (const epp_cs64* pSrc, epp_cs64* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cs64));
}
EPPINL(epp_Status, eppv_Copy_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cf16));
}
EPPINL(epp_Status, eppv_Copy_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cf32));
}
EPPINL(epp_Status, eppv_Copy_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length))
{
    return eppv_Copy_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cf64));
}


/*
//  Name:        eppv_Set
//  Purpose:     Sets vector elements by const value
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    val        The value to set.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Set_u8, (epp_u8 val, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Set_u16, (epp_u16 val, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Set_u32, (epp_u32 val, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Set_u64, (epp_u64 val, epp_u64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Set_cf64, (epp_cf64 val, epp_cf64* pDst, epp_size length))
EPPINL(epp_Status, eppv_Set_s8, (epp_s8 val, epp_s8* pDst, epp_size length))
{
    return eppv_Set_u8((epp_u8)val, (epp_u8*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_s16, (epp_s16 val, epp_s16* pDst, epp_size length))
{
    return eppv_Set_u16((epp_u16)val, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_s32, (epp_s32 val, epp_s32* pDst, epp_size length))
{
    return eppv_Set_u32((epp_u32)val, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_s64, (epp_s64 val, epp_s64* pDst, epp_size length))
{
    return eppv_Set_u64((epp_u64)val, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_f16, (epp_f16 val, epp_f16* pDst, epp_size length))
{
    return eppv_Set_u16((epp_u16)val, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_f32, (epp_f32 val, epp_f32* pDst, epp_size length))
{
    epp_u32* t = (epp_u32*)&val;
    return eppv_Set_u32(*t, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_f64, (epp_f64 val, epp_f64* pDst, epp_size length))
{
    epp_u64* t = (epp_u64*)&val;
    return eppv_Set_u64(*t, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_cs16, (epp_cs16 val, epp_cs16* pDst, epp_size length))
{
    epp_u32* t = (epp_u32*)&val;
    return eppv_Set_u32(*t, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_cs32, (epp_cs32 val, epp_cs32* pDst, epp_size length))
{
    epp_u64* t = (epp_u64*)&val;
    return eppv_Set_u64(*t, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_cs64, (epp_cs64 val, epp_cs64* pDst, epp_size length))
{
    epp_cf64* t = (epp_cf64*)&val;
    return eppv_Set_cf64(*t, (epp_cf64*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_cf16, (epp_cf16 val, epp_cf16* pDst, epp_size length))
{
    epp_u32* t = (epp_u32*)&val;
    return eppv_Set_u32(*t, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Set_cf32, (epp_cf32 val, epp_cf32* pDst, epp_size length))
{
    epp_u64* t = (epp_u64*)&val;
    return eppv_Set_u64(*t, (epp_u64*)pDst, length);
}

/*
//  Name:        eppv_Convert
//  Purpose:     Converts data
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsRoundModeErr  Unsupported round mode
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//    roundMode  The round mode.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Convert_u8u16, (const epp_u8* pSrc, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_u8f16, (const epp_u8* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_u8f32, (const epp_u8* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_f16s32, (const epp_f16* pSrc, epp_s32* pDst, epp_size length, epp_RoundMode roundMode))
EPPAPI(epp_Status, eppv_Convert_f16f32, (const epp_f16* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_f32f16, (const epp_f32* pSrc, epp_f16* pDst, epp_size length))
EPPINL(epp_Status, eppv_Convert_u8s16, (const epp_u8* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_Convert_u8u16(pSrc, (epp_u16*)pDst, length);
}
// todo implement
/*
EPPAPI(epp_Status, eppv_Convert_s16s32, (const epp_s16* pSrc, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_s16f16, (const epp_s16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_s16f32, (const epp_s16* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_s32f16, (const epp_s32* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_s32f16_S, (const epp_s32* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_s32f32, (const epp_s32* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Convert_f32f16, (const epp_f32* pSrc, epp_f16* pDst, epp_size length, epp_RoundMode roundMode))
EPPAPI(epp_Status, eppv_Convert_f32s32, (const epp_f32* pSrc, epp_s32* pDst, epp_size length, epp_RoundMode roundMode))
EPPAPI(epp_Status, eppv_Convert_f32f16_S, (const epp_f32* pSrc, epp_f16* pDst, epp_size length, epp_RoundMode roundMode))
EPPAPI(epp_Status, eppv_Convert_f32s32_S, (const epp_f32* pSrc, epp_s32* pDst, epp_size length, epp_RoundMode roundMode))
*/


/*
//  Name:        eppv_MulAdd
//  Purpose:     Multiplies elements of two vectors and adds to elements of third vector (d = s1 * s2 + s3)
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    pSrc2      The sourse vector2 data pointer.
//    pSrc3      The sourse vector3 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MulAdd_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, const epp_f16* pSrc3, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulAdd_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, const epp_f32* pSrc3, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulAdd_f64, (const epp_f64* pSrc1, const epp_f64* pSrc2, const epp_f64* pSrc3, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_MulCAdd
//  Purpose:     Multiplies elements of vector1 by value and adds to elements of second vector (d = s1 * v + s2)
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    vMul       The value to multiplicate first vector elements
//    pSrc2      The sourse vector2 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MulCAdd_s16_NS, (const epp_s16* pSrc1, epp_s16 vMul, const epp_s16* pSrc2, epp_s16* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_MulCAdd_s32_NS, (const epp_s32* pSrc1, epp_s32 vMul, const epp_s32* pSrc2, epp_s32* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_MulCAdd_f16, (const epp_f16* pSrc1, epp_f16 vMul, const epp_f16* pSrc2, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulCAdd_f32, (const epp_f32* pSrc1, epp_f32 vMul, const epp_f32* pSrc2, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulCAdd_f64, (const epp_f64* pSrc1, epp_f64 vMul, const epp_f64* pSrc2, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_MulCAddC
//  Purpose:     normilize data (dst = src * vMul + vAdd)
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    vMul       The value for multiplication.
//    vAdd       The value for addition.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MulCAddC_u8f16, (const epp_u8* pSrc, epp_f16 vMul, epp_f16 vAdd, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulCAddC_u8f32, (const epp_u8* pSrc, epp_f32 vMul, epp_f32 vAdd, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulCAddC_f16, (const epp_f16* pSrc, epp_f16 vMul, epp_f16 vAdd, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulCAddC_f32, (const epp_f32* pSrc, epp_f32 vMul, epp_f32 vAdd, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulCAddC_f64, (const epp_f64* pSrc, epp_f64 vMul, epp_f64 vAdd, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_Min, eppv_Max, eppv_MinMax
//  Purpose:     Finds min (and/or max) value of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pMin       min value.
//    pMin       max value.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Min_u8, (const epp_u8* pSrc, epp_size length, epp_u8* pMin))
EPPAPI(epp_Status, eppv_Min_s16, (const epp_s16* pSrc, epp_size length, epp_s16* pMin))
EPPAPI(epp_Status, eppv_Min_s32, (const epp_s32* pSrc, epp_size length, epp_s32* pMin))
EPPAPI(epp_Status, eppv_Min_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pMin))
EPPAPI(epp_Status, eppv_Min_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMin))
EPPAPI(epp_Status, eppv_Min_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMin))
EPPAPI(epp_Status, eppv_Max_u8, (const epp_u8* pSrc, epp_size length, epp_u8* pMax))
EPPAPI(epp_Status, eppv_Max_s16, (const epp_s16* pSrc, epp_size length, epp_s16* pMax))
EPPAPI(epp_Status, eppv_Max_s32, (const epp_s32* pSrc, epp_size length, epp_s32* pMax))
EPPAPI(epp_Status, eppv_Max_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pMax))
EPPAPI(epp_Status, eppv_Max_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMax))
EPPAPI(epp_Status, eppv_Max_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMax))
EPPAPI(epp_Status, eppv_MinMax_u8, (const epp_u8* pSrc, epp_size length, epp_u8* pMin, epp_u8* pMax))
EPPAPI(epp_Status, eppv_MinMax_s16, (const epp_s16* pSrc, epp_size length, epp_s16* pMin, epp_s16* pMax))
EPPAPI(epp_Status, eppv_MinMax_s32, (const epp_s32* pSrc, epp_size length, epp_s32* pMin, epp_s32* pMax))
EPPAPI(epp_Status, eppv_MinMax_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pMin, epp_f16* pMax))
EPPAPI(epp_Status, eppv_MinMax_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMin, epp_f32* pMax))
EPPAPI(epp_Status, eppv_MinMax_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMin, epp_f64* pMax))


/*
//  Name:        eppv_MinAbs, eppv_MaxAbs, eppv_MinMaxAbs
//  Purpose:     Finds min (and/or max) abs value of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pMin       min abs value.
//    pMax       max abs value.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MinAbs_s16, (const epp_s16* pSrc, epp_size length, epp_u16* pMin))
EPPAPI(epp_Status, eppv_MinAbs_s32, (const epp_s32* pSrc, epp_size length, epp_u32* pMin))
EPPAPI(epp_Status, eppv_MinAbs_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pMin))
EPPAPI(epp_Status, eppv_MinAbs_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMin))
EPPAPI(epp_Status, eppv_MinAbs_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMin))
EPPAPI(epp_Status, eppv_MaxAbs_s16, (const epp_s16* pSrc, epp_size length, epp_u16* pMin))
EPPAPI(epp_Status, eppv_MaxAbs_s32, (const epp_s32* pSrc, epp_size length, epp_u32* pMin))
EPPAPI(epp_Status, eppv_MaxAbs_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pMin))
EPPAPI(epp_Status, eppv_MaxAbs_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMin))
EPPAPI(epp_Status, eppv_MaxAbs_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMin))
EPPAPI(epp_Status, eppv_MinMaxAbs_s16, (const epp_s16* pSrc, epp_size length, epp_u16* pMin, epp_u16* pMax))
EPPAPI(epp_Status, eppv_MinMaxAbs_s32, (const epp_s32* pSrc, epp_size length, epp_u32* pMin, epp_u32* pMax))
EPPAPI(epp_Status, eppv_MinMaxAbs_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pMin, epp_f16* pMax))
EPPAPI(epp_Status, eppv_MinMaxAbs_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMin, epp_f32* pMax))
EPPAPI(epp_Status, eppv_MinMaxAbs_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMin, epp_f64* pMax))


/*
//  Name:        eppv_MinAbsDiff, eppv_MaxAbsDiff, eppv_MinMaxAbsDiff
//  Purpose:     Finds min (and/or max) abs value of difference of two vectors
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc1      The first sourse vector data pointer.
//    pSrc2      The second sourse vector data pointer.
//    length     The length of vector.
//    pMin       min abs value of difference.
//    pMax       max abs value of difference.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MaxAbsDiff_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f16* pMax))
EPPAPI(epp_Status, eppv_MaxAbsDiff_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pMax))


/*
//  Name:        eppv_Mul
//  Purpose:     Multiplicates vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    pSrc2      The sourse vector2 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Mul_s16_N, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_Mul_s16_NS, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_Mul_s32_N, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_Mul_s32_NS, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_Mul_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Mul_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Mul_f64, (const epp_f64* pSrc1, const epp_f64* pSrc2, epp_f64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Mul_cf16, (const epp_cf16* pSrc1, const epp_cf16* pSrc2, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Mul_cf32, (const epp_cf32* pSrc1, const epp_cf32* pSrc2, epp_cf32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Mul_cf64, (const epp_cf64* pSrc1, const epp_cf64* pSrc2, epp_cf64* pDst, epp_size length))


/*
//  Name:        eppv_MulConj
//  Purpose:     Multipliecats the elements of the first vector by conjugate of elements of the second vector (dst[i] = src1[i] * Conj(src2[i]))
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    pSrc2      The sourse vector2 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MulConj_cf16, (const epp_cf16* pSrc1, const epp_cf16* pSrc2, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulConj_cf32, (const epp_cf32* pSrc1, const epp_cf32* pSrc2, epp_cf32* pDst, epp_size length))


/*
//  Name:        eppv_MulC
//  Purpose:     Multiplicates vectors elements by value
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    val        The value to multiply.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_MulC_s16_N, (const epp_s16* pSrc, epp_s16 val, epp_s16* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_MulC_s16_NS, (const epp_s16* pSrc, epp_s16 val, epp_s16* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_MulC_s32_N, (const epp_s32* pSrc, epp_s32 val, epp_s32* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_MulC_s32_NS, (const epp_s32* pSrc, epp_s32 val, epp_s32* pDst, epp_size length, epp_size scale))
EPPAPI(epp_Status, eppv_MulC_f16, (const epp_f16* pSrc, epp_f16 val, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulC_f32, (const epp_f32* pSrc, epp_f32 val, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulC_f64, (const epp_f64* pSrc, epp_f64 val, epp_f64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulC_cf16, (const epp_cf16* pSrc, epp_cf16 val, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulC_cf32, (const epp_cf32* pSrc, epp_cf32 val, epp_cf32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_MulC_cf64, (const epp_cf64* pSrc, epp_cf64 val, epp_cf64* pDst, epp_size length))


/*
//  Name:        eppv_Div
//  Purpose:     Divides vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    pSrc2      The sourse vector2 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Div_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Div_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Add
//  Purpose:     Adds vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    pSrc2      The sourse vector2 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Add_s16, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Add_s16_S, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Add_s32, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Add_s32_S, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Add_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Add_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Add_f64, (const epp_f64* pSrc1, const epp_f64* pSrc2, epp_f64* pDst, epp_size length))
EPPINL(epp_Status, eppv_Add_cs16, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_Add_s16((const epp_s16*)pSrc1, (const epp_s16*)pSrc2, (epp_s16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Add_cs16_S, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_Add_s16_S((const epp_s16*)pSrc1, (const epp_s16*)pSrc2, (epp_s16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Add_cs32, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_Add_s32((const epp_s32*)pSrc1, (const epp_s32*)pSrc2, (epp_s32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Add_cs32_S, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_Add_s32_S((const epp_s32*)pSrc1, (const epp_s32*)pSrc2, (epp_s32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Add_cf16, (const epp_cf16* pSrc1, const epp_cf16* pSrc2, epp_cf16* pDst, epp_size length))
{
    return eppv_Add_f16((const epp_f16*)pSrc1, (const epp_f16*)pSrc2, (epp_f16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Add_cf32, (const epp_cf32* pSrc1, const epp_cf32* pSrc2, epp_cf32* pDst, epp_size length))
{
    return eppv_Add_f32((const epp_f32*)pSrc1, (const epp_f32*)pSrc2, (epp_f32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Add_cf64, (const epp_cf64* pSrc1, const epp_cf64* pSrc2, epp_cf64* pDst, epp_size length))
{
    return eppv_Add_f64((const epp_f64*)pSrc1, (const epp_f64*)pSrc2, (epp_f64*)pDst, length * 2);
}

/*
//  Name:        eppv_AddC
//  Purpose:     Adds vector elemets to const
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    cAdd       The const value.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_AddC_s16, (const epp_s16* pSrc, epp_s16 cAdd, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AddC_s16_S, (const epp_s16* pSrc, epp_s16 cAdd, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AddC_s32, (const epp_s32* pSrc, epp_s32 cAdd, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AddC_s32_S, (const epp_s32* pSrc, epp_s32 cAdd, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AddC_f16, (const epp_f16* pSrc, epp_f16 cAdd, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AddC_f32, (const epp_f32* pSrc, epp_f32 cAdd, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AddC_f64, (const epp_f64* pSrc, epp_f64 cAdd, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_Sub
//  Purpose:     Subtracs vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector1 data pointer.
//    pSrc2      The sourse vector2 data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Sub_s16, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sub_s16_S, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sub_s32, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sub_s32_S, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sub_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sub_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sub_f64, (const epp_f64* pSrc1, const epp_f64* pSrc2, epp_f64* pDst, epp_size length))
EPPINL(epp_Status, eppv_Sub_cs16, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_Sub_s16((const epp_s16*)pSrc1, (const epp_s16*)pSrc2, (epp_s16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Sub_cs16_S, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_Sub_s16_S((const epp_s16*)pSrc1, (const epp_s16*)pSrc2, (epp_s16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Sub_cs32, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_Sub_s32((const epp_s32*)pSrc1, (const epp_s32*)pSrc2, (epp_s32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Sub_cs32_S, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_Sub_s32_S((const epp_s32*)pSrc1, (const epp_s32*)pSrc2, (epp_s32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Sub_cf16, (const epp_cf16* pSrc1, const epp_cf16* pSrc2, epp_cf16* pDst, epp_size length))
{
    return eppv_Sub_f16((const epp_f16*)pSrc1, (const epp_f16*)pSrc2, (epp_f16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Sub_cf32, (const epp_cf32* pSrc1, const epp_cf32* pSrc2, epp_cf32* pDst, epp_size length))
{
    return eppv_Sub_f32((const epp_f32*)pSrc1, (const epp_f32*)pSrc2, (epp_f32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_Sub_cf64, (const epp_cf64* pSrc1, const epp_cf64* pSrc2, epp_cf64* pDst, epp_size length))
{
    return eppv_Sub_f64((const epp_f64*)pSrc1, (const epp_f64*)pSrc2, (epp_f64*)pDst, length * 2);
}


/*
//  Name:        eppv_CSub
//  Purpose:     Subtracs vector elemets from const
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    cSub       The const value.
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_CSub_s16, (epp_s16 cSub, const epp_s16* pSrc, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_CSub_s16_S, (epp_s16 cSub, const epp_s16* pSrc, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_CSub_s32, (epp_s32 cSub, const epp_s32* pSrc, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_CSub_s32_S, (epp_s32 cSub, const epp_s32* pSrc, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_CSub_f16, (epp_f16 cSub, const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_CSub_f32, (epp_f32 cSub, const epp_f32* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_CSub_f64, (epp_f64 cSub, const epp_f64* pSrc, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_Neg
//  Purpose:     Negatives vector elemets
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPINL(epp_Status, eppv_Neg_s16, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_CSub_s16(0, pSrc, pDst, length);
}
EPPINL(epp_Status, eppv_Neg_s16_S, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_CSub_s16_S(0, pSrc, pDst, length);
}
EPPINL(epp_Status, eppv_Neg_s32, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_CSub_s32(0, pSrc, pDst, length);
}
EPPINL(epp_Status, eppv_Neg_s32_S, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_CSub_s32_S(0, pSrc, pDst, length);
}
EPPINL(epp_Status, eppv_Neg_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
{
    return eppv_CSub_f16(0, pSrc, pDst, length);
}
EPPINL(epp_Status, eppv_Neg_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
{
    return eppv_CSub_f32(0, pSrc, pDst, length);
}
EPPINL(epp_Status, eppv_Neg_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))
{
    return eppv_CSub_f64(0, pSrc, pDst, length);
}


/*
//  Name:        eppv_Sum
//  Purpose:     Calculates the sum of vector elements
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Sum_s16, (const epp_s16* pSrc, epp_size length, epp_s64* pSum))
EPPAPI(epp_Status, eppv_Sum_s32, (const epp_s32* pSrc, epp_size length, epp_s64* pSum))
EPPAPI(epp_Status, eppv_Sum_f16, (const epp_f16* pSrc, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_Sum_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_Sum_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pSum))
EPPAPI(epp_Status, eppv_Sum_cf16, (const epp_cf16* pSrc, epp_size length, epp_cf32* pSum))
EPPAPI(epp_Status, eppv_Sum_cf32, (const epp_cf32* pSrc, epp_size length, epp_cf32* pSum))
EPPAPI(epp_Status, eppv_Sum_cf64, (const epp_cf64* pSrc, epp_size length, epp_cf64* pSum))


/*
//  Name:        eppv_SumAbs
//  Purpose:     Calculates the sum of vector absolute elements
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SumAbs_s16, (const epp_s16* pSrc, epp_size length, epp_u64* pSum))
EPPAPI(epp_Status, eppv_SumAbs_s32, (const epp_s32* pSrc, epp_size length, epp_u64* pSum))
EPPAPI(epp_Status, eppv_SumAbs_f16, (const epp_f16* pSrc, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumAbs_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumAbs_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pSum))


/*
//  Name:        eppv_SumSqr
//  Purpose:     Calculates the sum of squares of vector elements
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SumSqr_s16, (const epp_s16* pSrc, epp_size length, epp_u64* pSum))
EPPAPI(epp_Status, eppv_SumSqr_s32_N, (const epp_s32* pSrc, epp_size length, epp_u64* pSum, epp_size scale))
EPPAPI(epp_Status, eppv_SumSqr_f16, (const epp_f16* pSrc, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumSqr_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumSqr_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pSum))


/*
//  Name:        eppv_SumSqrAbs
//  Purpose:     Calculates the sum of squares of absolute values of vector elements
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPINL(epp_Status, eppv_SumSqrAbs_cs16, (const epp_cs16* pSrc, epp_size length, epp_u64* pSum))
{
    return eppv_SumSqr_s16((const epp_s16*)pSrc, length * 2, pSum);
}
EPPINL(epp_Status, eppv_SumSqrAbs_cs32_N, (const epp_cs32* pSrc, epp_size length, epp_u64* pSum, epp_size scale))
{
    return eppv_SumSqr_s32_N((const epp_s32*)pSrc, length * 2, pSum, scale);
}
EPPINL(epp_Status, eppv_SumSqrAbs_cf16, (const epp_cf16* pSrc, epp_size length, epp_f32* pSum))
{
    return eppv_SumSqr_f16((const epp_f16*)pSrc, length * 2, pSum);
}
EPPINL(epp_Status, eppv_SumSqrAbs_cf32, (const epp_cf32* pSrc, epp_size length, epp_f32* pSum))
{
    return eppv_SumSqr_f32((const epp_f32*)pSrc, length * 2, pSum);
}
EPPINL(epp_Status, eppv_SumSqrAbs_cf64, (const epp_cf64* pSrc, epp_size length, epp_f64* pSum))
{
    return eppv_SumSqr_f64((const epp_f64*)pSrc, length * 2, pSum);
}


/*
//  Name:        eppv_SumAbsDiff
//  Purpose:     Calculates the sum of absolute values of differences of elements of two vectors
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc1      The first sourse vector data pointer.
//    pSrc2      The second sourse vector data pointer.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SumAbsDiff_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumAbsDiff_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pSum))


/*
//  Name:        eppv_SumSqrDiff
//  Purpose:     Calculates the sum of squared differences of elements of two vectors
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc1      The first sourse vector data pointer.
//    pSrc2      The second sourse vector data pointer.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SumSqrDiff_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumSqrDiff_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pSum))


/*
//  Name:        eppv_SumSqrDiffC
//  Purpose:     Calculates the sum of squared differences of vectror elements and const value.
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    val        The const value.
//    length     The length of vector.
//    pSum       The sum of values.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SumSqrDiffC_f16, (const epp_f16* pSrc, epp_f16 val, epp_size length, epp_f32* pSum))
EPPAPI(epp_Status, eppv_SumSqrDiffC_f32, (const epp_f32* pSrc, epp_f32 val, epp_size length, epp_f32* pSum))


/*
//  Name:        eppv_DotProd
//  Purpose:     Calculates the dot product of two vectors
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc1      The sourse vector data pointer.
//    pSrc2      The sourse vector data pointer.
//    length     The length of vector.
//    pDp        The pointer to result.
//    scale      The scale factor.
//  Notes:
*/
EPPAPI(epp_Status, eppv_DotProd_s16, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_size length, epp_s64* pDp))
EPPAPI(epp_Status, eppv_DotProd_s32_N, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_size length, epp_s64* pDp, epp_size scale))
EPPAPI(epp_Status, eppv_DotProd_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f32* pDp))
EPPAPI(epp_Status, eppv_DotProd_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pDp))
EPPAPI(epp_Status, eppv_DotProd_f64, (const epp_f64* pSrc1, const epp_f64* pSrc2, epp_size length, epp_f64* pDp))
EPPAPI(epp_Status, eppv_DotProd_cf16, (const epp_cf16* pSrc1, const epp_cf16* pSrc2, epp_size length, epp_cf32* pDp))
EPPAPI(epp_Status, eppv_DotProd_cf32, (const epp_cf32* pSrc1, const epp_cf32* pSrc2, epp_size length, epp_cf32* pDp))
EPPAPI(epp_Status, eppv_DotProd_cf64, (const epp_cf64* pSrc1, const epp_cf64* pSrc2, epp_size length, epp_cf64* pDp))


/*
//  Name:        eppv_Abs
//  Purpose:     Calculates abs value of vector elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Abs_s16_S, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Abs_s16u16, (const epp_s16* pSrc, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Abs_s32_S, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Abs_s32u32, (const epp_s32* pSrc, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Abs_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Abs_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Abs_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))
// todo f16, f32, f64 - EPPINL via eppv_AndC


/*
//  Name:        eppv_Sqr
//  Purpose:     Calculates squared values of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Sqr_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sqr_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sqr_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_SqrAbs
//  Purpose:     Calculates squared absolute values of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SqrAbs_cf16, (const epp_cf16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_SqrAbs_cf32, (const epp_cf32* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_SqrAbs_cf64, (const epp_cf64* pSrc, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_Sqrt
//  Purpose:     Calculates square root of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Sqrt_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sqrt_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Exp
//  Purpose:     Calculates exp of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Exp_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Exp_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Exp_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Exp_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))


/*
//  Name:        eppv_Log
//  Purpose:     Calculates log of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Log_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Log_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Sin
//  Purpose:     Calculates sin of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Sin_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sin_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Cos
//  Purpose:     Calculates cos of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Cos_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Cos_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Tan
//  Purpose:     Calculates tangent of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Tan_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Tan_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Asin
//  Purpose:     Calculates arcsin of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Asin_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Asin_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Acos
//  Purpose:     Calculates arccos of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Acos_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Acos_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Atan
//  Purpose:     Calculates arctan of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Atan_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Atan_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Atan2
//  Purpose:     Calculates atan2 of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrcIm       The sourse image vector data pointer.
//    pSrcRe       The sourse real vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Atan2_f32, (const epp_f32* pSrcIm, const epp_f32* pSrcRe, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Atan2_cf32f32, (const epp_cf32* pSrc, epp_f32* pDst, epp_size lengt))


/*
//  Name:        eppv_Sinh
//  Purpose:     Calculates sinh of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Sinh_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Sinh_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Cosh
//  Purpose:     Calculates cosh of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Cosh_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Cosh_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Tanh
//  Purpose:     Calculates hyperbolic tangent of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Tanh_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Tanh_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Asinh
//  Purpose:     Calculates asinh of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Asinh_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Asinh_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Acosh
//  Purpose:     Calculates acosh of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Acosh_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Acosh_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Atanh
//  Purpose:     Calculates inverse hyperbolic tangent of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Atanh_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Atanh_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_Pow
//  Purpose:     Calculates pow of vectors elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1       The sourse vector data pointer.
//    pSrc2       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Pow_f16, (const epp_f16* pSrc1,const epp_f16* pSrc2, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Pow_f32, (const epp_f32* pSrc1,const epp_f32* pSrc2, epp_f32* pDst, epp_size length))


/*
//  Name:       eppv_PowC
//  Purpose:    Computes src raised to the val_th power. (dst = src^val)
//  Context:
//  Returns:    epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    val        The value to pow.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_PowC_f16, (const epp_f16* pSrc, epp_f16 val, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_PowC_f32, (const epp_f32* pSrc, epp_f32 val, epp_f32* pDst, epp_size length))


/*
//  Name:        eppv_And/Or/Xor/Not
//  Purpose:     Computes the bitwise And/Or/Xor/Not of vectors
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc1      The sourse vector data pointer.
//    pSrc2      The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_And_u8, (const epp_u8* pSrc1, const epp_u8* pSrc2, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Or_u8, (const epp_u8* pSrc1, const epp_u8* pSrc2, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Xor_u8, (const epp_u8* pSrc1, const epp_u8* pSrc2, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Not_u8, (const epp_u8* pSrc, epp_u8* pDst, epp_size length))
EPPINL(epp_Status, eppv_And_s8, (const epp_s8* pSrc1, const epp_s8* pSrc2, epp_s8* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s8));
}
EPPINL(epp_Status, eppv_And_s16, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s16));
}
EPPINL(epp_Status, eppv_And_u16, (const epp_u16* pSrc1, const epp_u16* pSrc2, epp_u16* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u16));
}
EPPINL(epp_Status, eppv_And_s32, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s32));
}
EPPINL(epp_Status, eppv_And_u32, (const epp_u32* pSrc1, const epp_u32* pSrc2, epp_u32* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u32));
}
EPPINL(epp_Status, eppv_And_s64, (const epp_s64* pSrc1, const epp_s64* pSrc2, epp_s64* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s64));
}
EPPINL(epp_Status, eppv_And_u64, (const epp_u64* pSrc1, const epp_u64* pSrc2, epp_u64* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u64));
}
EPPINL(epp_Status, eppv_And_cs16, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_cs16));
}
EPPINL(epp_Status, eppv_And_cs32, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_And_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_cs32));
}
EPPINL(epp_Status, eppv_Or_s8, (const epp_s8* pSrc1, const epp_s8* pSrc2, epp_s8* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s8));
}
EPPINL(epp_Status, eppv_Or_s16, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s16));
}
EPPINL(epp_Status, eppv_Or_u16, (const epp_u16* pSrc1, const epp_u16* pSrc2, epp_u16* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u16));
}
EPPINL(epp_Status, eppv_Or_s32, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s32));
}
EPPINL(epp_Status, eppv_Or_u32, (const epp_u32* pSrc1, const epp_u32* pSrc2, epp_u32* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u32));
}
EPPINL(epp_Status, eppv_Or_s64, (const epp_s64* pSrc1, const epp_s64* pSrc2, epp_s64* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s64));
}
EPPINL(epp_Status, eppv_Or_u64, (const epp_u64* pSrc1, const epp_u64* pSrc2, epp_u64* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u64));
}
EPPINL(epp_Status, eppv_Or_cs16, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_cs16));
}
EPPINL(epp_Status, eppv_Or_cs32, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_Or_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_cs32));
}
EPPINL(epp_Status, eppv_Xor_s8, (const epp_s8* pSrc1, const epp_s8* pSrc2, epp_s8* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s8));
}
EPPINL(epp_Status, eppv_Xor_s16, (const epp_s16* pSrc1, const epp_s16* pSrc2, epp_s16* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s16));
}
EPPINL(epp_Status, eppv_Xor_u16, (const epp_u16* pSrc1, const epp_u16* pSrc2, epp_u16* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u16));
}
EPPINL(epp_Status, eppv_Xor_s32, (const epp_s32* pSrc1, const epp_s32* pSrc2, epp_s32* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s32));
}
EPPINL(epp_Status, eppv_Xor_u32, (const epp_u32* pSrc1, const epp_u32* pSrc2, epp_u32* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u32));
}
EPPINL(epp_Status, eppv_Xor_s64, (const epp_s64* pSrc1, const epp_s64* pSrc2, epp_s64* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_s64));
}
EPPINL(epp_Status, eppv_Xor_u64, (const epp_u64* pSrc1, const epp_u64* pSrc2, epp_u64* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_u64));
}
EPPINL(epp_Status, eppv_Xor_cs16, (const epp_cs16* pSrc1, const epp_cs16* pSrc2, epp_cs16* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_cs16));
}
EPPINL(epp_Status, eppv_Xor_cs32, (const epp_cs32* pSrc1, const epp_cs32* pSrc2, epp_cs32* pDst, epp_size length))
{
    return eppv_Xor_u8((const epp_u8*)pSrc1, (const epp_u8*)pSrc2, (epp_u8*)pDst, length * sizeof(epp_cs32));
}
EPPINL(epp_Status, eppv_Not_s8, (const epp_s8* pSrc, epp_s8* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s8));
}
EPPINL(epp_Status, eppv_Not_s16, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s16));
}
EPPINL(epp_Status, eppv_Not_u16, (const epp_u16* pSrc, epp_u16* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_u16));
}
EPPINL(epp_Status, eppv_Not_s32, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s32));
}
EPPINL(epp_Status, eppv_Not_u32, (const epp_u32* pSrc, epp_u32* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_u32));
}
EPPINL(epp_Status, eppv_Not_s64, (const epp_s64* pSrc, epp_s64* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_s64));
}
EPPINL(epp_Status, eppv_Not_u64, (const epp_u64* pSrc, epp_u64* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_u64));
}
EPPINL(epp_Status, eppv_Not_cs16, (const epp_cs16* pSrc, epp_cs16* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cs16));
}
EPPINL(epp_Status, eppv_Not_cs32, (const epp_cs32* pSrc, epp_cs32* pDst, epp_size length))
{
    return eppv_Not_u8((const epp_u8*)pSrc, (epp_u8*)pDst, length * sizeof(epp_cs32));
}


/*
//  Name:        eppv_AndC/OrC/XorC
//  Purpose:     Computes the bitwise And/Or/Xor/Not of vector and value
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    val        The const value.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_AndC_u8, (const epp_u8* pSrc, epp_u8 val, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AndC_u16, (const epp_u16* pSrc, epp_u16 val, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AndC_u32, (const epp_u32* pSrc, epp_u32 val, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_AndC_u64, (const epp_u64* pSrc, epp_u64 val, epp_u64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_OrC_u8, (const epp_u8* pSrc, epp_u8 val, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_OrC_u16, (const epp_u16* pSrc, epp_u16 val, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_OrC_u32, (const epp_u32* pSrc, epp_u32 val, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_OrC_u64, (const epp_u64* pSrc, epp_u64 val, epp_u64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_XorC_u8, (const epp_u8* pSrc, epp_u8 val, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_XorC_u16, (const epp_u16* pSrc, epp_u16 val, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_XorC_u32, (const epp_u32* pSrc, epp_u32 val, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_XorC_u64, (const epp_u64* pSrc, epp_u64 val, epp_u64* pDst, epp_size length))
EPPINL(epp_Status, eppv_AndC_s8, (const epp_s8* pSrc, epp_s8 val, epp_s8* pDst, epp_size length))
{
    return eppv_AndC_u8((const epp_u8*)pSrc, (epp_u8)val, (epp_u8*)pDst, length);
}
EPPINL(epp_Status, eppv_AndC_s16, (const epp_s16* pSrc, epp_s16 val, epp_s16* pDst, epp_size length))
{
    return eppv_AndC_u16((const epp_u16*)pSrc, (epp_u16)val, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_AndC_s32, (const epp_s32* pSrc, epp_s32 val, epp_s32* pDst, epp_size length))
{
    return eppv_AndC_u32((const epp_u32*)pSrc, (epp_u32)val, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_AndC_s64, (const epp_s64* pSrc, epp_s64 val, epp_s64* pDst, epp_size length))
{
    return eppv_AndC_u64((const epp_u64*)pSrc, (epp_u64)val, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_AndC_cs16, (const epp_cs16* pSrc, epp_cs16 val, epp_cs16* pDst, epp_size length))
{
    epp_u32* t = (epp_u32*)&val;
    return eppv_AndC_u32((const epp_u32*)pSrc, *t, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_AndC_cs32, (const epp_cs32* pSrc, epp_cs32 val, epp_cs32* pDst, epp_size length))
{
    epp_u64* t = (epp_u64*)&val;
    return eppv_AndC_u64((const epp_u64*)pSrc, *t, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_OrC_s8, (const epp_s8* pSrc, epp_s8 val, epp_s8* pDst, epp_size length))
{
    return eppv_OrC_u8((const epp_u8*)pSrc, (epp_u8)val, (epp_u8*)pDst, length);
}
EPPINL(epp_Status, eppv_OrC_s16, (const epp_s16* pSrc, epp_s16 val, epp_s16* pDst, epp_size length))
{
    return eppv_OrC_u16((const epp_u16*)pSrc, (epp_u16)val, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_OrC_s32, (const epp_s32* pSrc, epp_s32 val, epp_s32* pDst, epp_size length))
{
    return eppv_OrC_u32((const epp_u32*)pSrc, (epp_u32)val, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_OrC_s64, (const epp_s64* pSrc, epp_s64 val, epp_s64* pDst, epp_size length))
{
    return eppv_OrC_u64((const epp_u64*)pSrc, (epp_u64)val, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_OrC_cs16, (const epp_cs16* pSrc, epp_cs16 val, epp_cs16* pDst, epp_size length))
{
    epp_u32* t = (epp_u32*)&val;
    return eppv_OrC_u32((const epp_u32*)pSrc, *t, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_OrC_cs32, (const epp_cs32* pSrc, epp_cs32 val, epp_cs32* pDst, epp_size length))
{
    epp_u64* t = (epp_u64*)&val;
    return eppv_OrC_u64((const epp_u64*)pSrc, *t, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_XorC_s8, (const epp_s8* pSrc, epp_s8 val, epp_s8* pDst, epp_size length))
{
    return eppv_XorC_u8((const epp_u8*)pSrc, (epp_u8)val, (epp_u8*)pDst, length);
}
EPPINL(epp_Status, eppv_XorC_s16, (const epp_s16* pSrc, epp_s16 val, epp_s16* pDst, epp_size length))
{
    return eppv_XorC_u16((const epp_u16*)pSrc, (epp_u16)val, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_XorC_s32, (const epp_s32* pSrc, epp_s32 val, epp_s32* pDst, epp_size length))
{
    return eppv_XorC_u32((const epp_u32*)pSrc, (epp_u32)val, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_XorC_s64, (const epp_s64* pSrc, epp_s64 val, epp_s64* pDst, epp_size length))
{
    return eppv_XorC_u64((const epp_u64*)pSrc, (epp_u64)val, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_XorC_cs16, (const epp_cs16* pSrc, epp_cs16 val, epp_cs16* pDst, epp_size length))
{
    epp_u32* t = (epp_u32*)&val;
    return eppv_XorC_u32((const epp_u32*)pSrc, *t, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_XorC_cs32, (const epp_cs32* pSrc, epp_cs32 val, epp_cs32* pDst, epp_size length))
{
    epp_u64* t = (epp_u64*)&val;
    return eppv_XorC_u64((const epp_u64*)pSrc, *t, (epp_u64*)pDst, length);
}


/*
//  Name:        eppv_ShiftLC/ShiftRC
//  Purpose:     Computes the logical and arithmetic bit shift of vector's elements
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    shift      The shift value.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_ShiftLC_u8, (const epp_u8* pSrc, epp_size shift, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftLC_u16, (const epp_u16* pSrc, epp_size shift, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftLC_u32, (const epp_u32* pSrc, epp_size shift, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftLC_u64, (const epp_u64* pSrc, epp_size shift, epp_u64* pDst, epp_size length))
EPPINL(epp_Status, eppv_ShiftLC_s8, (const epp_s8* pSrc, epp_size shift, epp_s8* pDst, epp_size length))
{
    return eppv_ShiftLC_u8((const epp_u8*)pSrc, shift, (epp_u8*)pDst, length);
}
EPPINL(epp_Status, eppv_ShiftLC_s16, (const epp_s16* pSrc, epp_size shift, epp_s16* pDst, epp_size length))
{
    return eppv_ShiftLC_u16((const epp_u16*)pSrc, shift, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_ShiftLC_s32, (const epp_s32* pSrc, epp_size shift, epp_s32* pDst, epp_size length))
{
    return eppv_ShiftLC_u32((const epp_u32*)pSrc, shift, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_ShiftLC_s64, (const epp_s64* pSrc, epp_size shift, epp_s64* pDst, epp_size length))
{
    return eppv_ShiftLC_u64((const epp_u64*)pSrc, shift, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_ShiftLC_cs16, (const epp_cs16* pSrc, epp_size shift, epp_cs16* pDst, epp_size length))
{
    return eppv_ShiftLC_u16((const epp_u16*)pSrc, shift, (epp_u16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_ShiftLC_cs32, (const epp_cs32* pSrc, epp_size shift, epp_cs32* pDst, epp_size length))
{
    return eppv_ShiftLC_u32((const epp_u32*)pSrc, shift, (epp_u32*)pDst, length * 2);
}
EPPAPI(epp_Status, eppv_ShiftRC_u8, (const epp_u8* pSrc, epp_size shift, epp_u8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_s8, (const epp_s8* pSrc, epp_size shift, epp_s8* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_u16, (const epp_u16* pSrc, epp_size shift, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_s16, (const epp_s16* pSrc, epp_size shift, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_u32, (const epp_u32* pSrc, epp_size shift, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_s32, (const epp_s32* pSrc, epp_size shift, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_u64, (const epp_u64* pSrc, epp_size shift, epp_u64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ShiftRC_s64, (const epp_s64* pSrc, epp_size shift, epp_s64* pDst, epp_size length))
EPPINL(epp_Status, eppv_ShiftRC_cs16, (const epp_cs16* pSrc, epp_size shift, epp_cs16* pDst, epp_size length))
{
    return eppv_ShiftRC_s16((const epp_s16*)pSrc, shift, (epp_s16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_ShiftRC_cs32, (const epp_cs32* pSrc, epp_size shift, epp_cs32* pDst, epp_size length))
{
    return eppv_ShiftRC_s32((const epp_s32*)pSrc, shift, (epp_s32*)pDst, length * 2);
}


/*
//  Name:        eppv_Round
//  Purpose:     Computes rounded values of vector elements
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsRoundModeErr  Unsupported round mode
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//    roundMode  The round mode.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Round_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length, epp_RoundMode roundMode))
EPPAPI(epp_Status, eppv_Round_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length, epp_RoundMode roundMode))
EPPAPI(epp_Status, eppv_Round_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length, epp_RoundMode roundMode))
EPPINL(epp_Status, eppv_Round_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length, epp_RoundMode roundMode))
{
    return eppv_Round_f16((const epp_f16*)pSrc, (epp_f16*)pDst, length * 2, roundMode);
}
EPPINL(epp_Status, eppv_Round_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length, epp_RoundMode roundMode))
{
    return eppv_Round_f32((const epp_f32*)pSrc, (epp_f32*)pDst, length * 2, roundMode);
}
EPPINL(epp_Status, eppv_Round_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length, epp_RoundMode roundMode))
{
    return eppv_Round_f64((const epp_f64*)pSrc, (epp_f64*)pDst, length * 2, roundMode);
}


/*
//  Name:        eppv_NormC/L1/L2
//  Purpose:     Calculates the norm of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pNorm      The resulting norm.
//  Notes:
*/
EPPINL(epp_Status, eppv_NormC_s16, (const epp_s16* pSrc, epp_size length, epp_u16* pNorm))
{
    return eppv_MaxAbs_s16(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormC_s32, (const epp_s32* pSrc, epp_size length, epp_u32* pNorm))
{
    return eppv_MaxAbs_s32(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormC_f16, (const epp_f16* pSrc, epp_size length, epp_f16* pNorm))
{
    return eppv_MaxAbs_f16(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormC_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pNorm))
{
    return eppv_MaxAbs_f32(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormC_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pNorm))
{
    return eppv_MaxAbs_f64(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormL1_s16, (const epp_s16* pSrc, epp_size length, epp_u64* pNorm))
{
    return eppv_SumAbs_s16(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormL1_s32, (const epp_s32* pSrc, epp_size length, epp_u64* pNorm))
{
    return eppv_SumAbs_s32(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormL1_f16, (const epp_f16* pSrc, epp_size length, epp_f32* pNorm))
{
    return eppv_SumAbs_f16(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormL1_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pNorm))
{
    return eppv_SumAbs_f32(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormL1_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pNorm))
{
    return eppv_SumAbs_f64(pSrc, length, pNorm);
}
EPPINL(epp_Status, eppv_NormL2_s16, (const epp_s16* pSrc, epp_size length, epp_f32* pNorm))
{
    epp_u64 ss;
    epp_Status sts = eppv_SumSqr_s16(pSrc, length, &ss);
    *pNorm = (epp_f32)sqrt(ss);
    return sts;
}
EPPINL(epp_Status, eppv_NormL2_s32_N, (const epp_s32* pSrc, epp_size length, epp_f32* pNorm, epp_size scale))
{
    epp_u64 ss;
    epp_Status sts = eppv_SumSqr_s32_N(pSrc, length, &ss, scale);
    *pNorm = (epp_f32)sqrt(ss);
    return sts;
}
EPPINL(epp_Status, eppv_NormL2_f16, (const epp_f16* pSrc, epp_size length, epp_f32* pNorm))
{
    epp_f32 ss;
    epp_Status sts = eppv_SumSqr_f16(pSrc, length, &ss);
    *pNorm = (epp_f32)sqrtf(ss);
    return sts;
}
EPPINL(epp_Status, eppv_NormL2_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pNorm))
{
    epp_f32 ss;
    epp_Status sts = eppv_SumSqr_f32(pSrc, length, &ss);
    *pNorm = (epp_f32)sqrtf(ss);
    return sts;
}
EPPINL(epp_Status, eppv_NormL2_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pNorm))
{
    epp_f64 ss;
    epp_Status sts = eppv_SumSqr_f64(pSrc, length, &ss);
    *pNorm = (epp_f64)sqrt(ss);
    return sts;
}


/*
//  Name:        eppv_NormDiffC/L1/L2
//  Purpose:     Calculates the C, L1, L2 norm of difference of two vectors
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc1      The first sourse vector data pointer.
//    pSrc2      The second sourse vector data pointer.
//    length     The length of vector.
//    pNorm      The norm of difference.
//  Notes:
*/
EPPINL(epp_Status, eppv_NormDiffC_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f16* pNorm))
{
    return eppv_MaxAbsDiff_f16(pSrc1, pSrc2, length, pNorm);
}
EPPINL(epp_Status, eppv_NormDiffC_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pNorm))
{
    return eppv_MaxAbsDiff_f32(pSrc1, pSrc2, length, pNorm);
}
EPPINL(epp_Status, eppv_NormDiffL1_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f32* pNorm))
{
    return eppv_SumAbsDiff_f16(pSrc1, pSrc2, length, pNorm);
}
EPPINL(epp_Status, eppv_NormDiffL1_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pNorm))
{
    return eppv_SumAbsDiff_f32(pSrc1, pSrc2, length, pNorm);
}
EPPINL(epp_Status, eppv_NormDiffL2_f16, (const epp_f16* pSrc1, const epp_f16* pSrc2, epp_size length, epp_f32* pNorm))
{
    epp_f32 ss;
    epp_Status sts = eppv_SumSqrDiff_f16(pSrc1, pSrc2, length, &ss);
    *pNorm = (epp_f32)sqrtf(ss);
    return sts;
}
EPPINL(epp_Status, eppv_NormDiffL2_f32, (const epp_f32* pSrc1, const epp_f32* pSrc2, epp_size length, epp_f32* pNorm))
{
    epp_f32 ss;
    epp_Status sts = eppv_SumSqrDiff_f32(pSrc1, pSrc2, length, &ss);
    *pNorm = (epp_f32)sqrtf(ss);
    return sts;
}


/*
//  Name:        eppv_Mean
//  Purpose:     Calculates the mean value of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    length     The length of vector.
//    pMean      The resulting value.
//  Notes:
*/
EPPINL(epp_Status, eppv_Mean_s16, (const epp_s16* pSrc, epp_size length, epp_f32* pMean))
{
    epp_s64 ss;
    epp_Status sts = eppv_Sum_s16(pSrc, length, &ss);
    *pMean = (epp_f32)ss / length;
    return sts;
}
EPPINL(epp_Status, eppv_Mean_s32, (const epp_s32* pSrc, epp_size length, epp_f32* pMean))
{
    epp_s64 ss;
    epp_Status sts = eppv_Sum_s32(pSrc, length, &ss);
    *pMean = (epp_f32)ss / length;
    return sts;
}
EPPINL(epp_Status, eppv_Mean_f16, (const epp_f16* pSrc, epp_size length, epp_f32* pMean))
{
    epp_f32 ss;
    epp_Status sts = eppv_Sum_f16(pSrc, length, &ss);
    *pMean = (epp_f32)ss / length;
    return sts;
}
EPPINL(epp_Status, eppv_Mean_f32, (const epp_f32* pSrc, epp_size length, epp_f32* pMean))
{
    epp_f32 ss;
    epp_Status sts = eppv_Sum_f32(pSrc, length, &ss);
    *pMean = (epp_f32)ss / length;
    return sts;
}
EPPINL(epp_Status, eppv_Mean_f64, (const epp_f64* pSrc, epp_size length, epp_f64* pMean))
{
    epp_f64 ss;
    epp_Status sts = eppv_Sum_f64(pSrc, length, &ss);
    *pMean = (epp_f64)ss / length;
    return sts;
}


/*
//  Name:        eppv_VarianceBiased
//  Purpose:     Calculates the biased variance of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    mean       The mean value of vector elements.
//    length     The length of vector.
//    pVar       The biased variance.
//  Notes:
*/
EPPINL(epp_Status, eppv_VarianceBiased_f16, (const epp_f16* pSrc, epp_f16 mean, epp_size length, epp_f32* pVar))
{
    epp_f32 ss;
    epp_Status sts = eppv_SumSqrDiffC_f16(pSrc, mean, length, &ss);
    *pVar = (epp_f32)(ss / length);
    return sts;
}
EPPINL(epp_Status, eppv_VarianceBiased_f32, (const epp_f32* pSrc, epp_f32 mean, epp_size length, epp_f32* pVar))
{
    epp_f32 ss;
    epp_Status sts = eppv_SumSqrDiffC_f32(pSrc, mean, length, &ss);
    *pVar = (epp_f32)(ss / length);
    return sts;
}


/*
//  Name:        eppv_VarianceUnbiased
//  Purpose:     Calculates the unbiased variance of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length < 2
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    mean       The mean value of vector elements.
//    length     The length of vector.
//    pVar       The unbiased variance.
//  Notes:
*/
EPPINL(epp_Status, eppv_VarianceUnbiased_f16, (const epp_f16* pSrc, epp_f16 mean, epp_size length, epp_f32* pVar))
{
    if (length < 2) {
        return epp_StsBadParamsErr;
    }
    epp_f32 ss;
    epp_Status sts = eppv_SumSqrDiffC_f16(pSrc, mean, length, &ss);
    *pVar = (epp_f32)(ss / (length  - 1));
    return sts;
}
EPPINL(epp_Status, eppv_VarianceUnbiased_f32, (const epp_f32* pSrc, epp_f32 mean, epp_size length, epp_f32* pVar))
{
    if (length < 2) {
        return epp_StsBadParamsErr;
    }
    epp_f32 ss;
    epp_Status sts = eppv_SumSqrDiffC_f32(pSrc, mean, length, &ss);
    *pVar = (epp_f32)(ss / (length  - 1));
    return sts;
}


/*
//  Name:        eppv_StdDevBiased
//  Purpose:     Calculates the biased standard deviation of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length == 0
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    mean       The mean value of vector elements.
//    length     The length of vector.
//    pDev       The biased standard deviation.
//  Notes:
*/
EPPINL(epp_Status, eppv_StdDevBiased_f16, (const epp_f16* pSrc, epp_f16 mean, epp_size length, epp_f32* pDev))
{
    epp_f32 var;
    epp_Status sts = eppv_VarianceBiased_f16(pSrc, mean, length, &var);
    *pDev = (epp_f32)sqrtf(var);
    return sts;
}
EPPINL(epp_Status, eppv_StdDevBiased_f32, (const epp_f32* pSrc, epp_f32 mean, epp_size length, epp_f32* pDev))
{
    epp_f32 var;
    epp_Status sts = eppv_VarianceBiased_f32(pSrc, mean, length, &var);
    *pDev = (epp_f32)sqrtf(var);
    return sts;
}


/*
//  Name:        eppv_StdDevUnbiased
//  Purpose:     Calculates the unbiased standard deviation of vector
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length < 2
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    mean       The mean value of vector elements.
//    length     The length of vector.
//    pDev       The unbiased standard deviation.
//  Notes:
*/
EPPINL(epp_Status, eppv_StdDevUnbiased_f16, (const epp_f16* pSrc, epp_f16 mean, epp_size length, epp_f32* pDev))
{
    epp_f32 var;
    epp_Status sts = eppv_VarianceUnbiased_f16(pSrc, mean, length, &var);
    *pDev = (epp_f32)sqrtf(var);
    return sts;
}
EPPINL(epp_Status, eppv_StdDevUnbiased_f32, (const epp_f32* pSrc, epp_f32 mean, epp_size length, epp_f32* pDev))
{
    epp_f32 var;
    epp_Status sts = eppv_VarianceUnbiased_f32(pSrc, mean, length, &var);
    *pDev = (epp_f32)sqrtf(var);
    return sts;
}


/*
//  Name:        eppv_Conj
//  Purpose:     Calculates conjugation of vector elements
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Conj_cs16_S, (const epp_cs16* pSrc, epp_cs16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Conj_cs32_S, (const epp_cs32* pSrc, epp_cs32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Conj_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Conj_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Conj_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length))


/*
//  Name:        eppv_ConjSym
//  Purpose:     Calculates conjugation of vector elements and store them in reverse order (conjugate symmetry)
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_ConjSym_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ConjSym_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ConjSym_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length))


/*
//  Name:        eppv_ThresholdLT
//  Purpose:     Limits vector elements by threshold (pDst[i] = (pSrc[i] < val) ? val : pSrc[i])
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    tVal       The threshold value.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_ThresholdLT_s16, (const epp_s16* pSrc, epp_s16 tVal, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdLT_s32, (const epp_s32* pSrc, epp_s32 tVal, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdLT_f16, (const epp_f16* pSrc, epp_f16 tVal, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdLT_f32, (const epp_f32* pSrc, epp_f32 tVal, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdLT_f64, (const epp_f64* pSrc, epp_f64 tVal, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_ThresholdGT
//  Purpose:     Limits vector elements by threshold (pDst[i] = (pSrc[i] > val) ? val : pSrc[i])
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    tVal       The threshold value.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_ThresholdGT_s16, (const epp_s16* pSrc, epp_s16 tVal, epp_s16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdGT_s32, (const epp_s32* pSrc, epp_s32 tVal, epp_s32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdGT_f16, (const epp_f16* pSrc, epp_f16 tVal, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdGT_f32, (const epp_f32* pSrc, epp_f32 tVal, epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_ThresholdGT_f64, (const epp_f64* pSrc, epp_f64 tVal, epp_f64* pDst, epp_size length))


/*
//  Name:        eppv_SwapBytes
//  Purpose:     Inverts the byte order of vector elements
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SwapBytes_u16, (const epp_u16* pSrc, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_SwapBytes_u32, (const epp_u32* pSrc, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_SwapBytes_u64, (const epp_u64* pSrc, epp_u64* pDst, epp_size length))
EPPINL(epp_Status, eppv_SwapBytes_s16, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_SwapBytes_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapBytes_s32, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_SwapBytes_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapBytes_s64, (const epp_s64* pSrc, epp_s64* pDst, epp_size length))
{
    return eppv_SwapBytes_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapBytes_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
{
    return eppv_SwapBytes_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapBytes_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
{
    return eppv_SwapBytes_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapBytes_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))
{
    return eppv_SwapBytes_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapBytes_cs16, (const epp_cs16* pSrc, epp_cs16* pDst, epp_size length))
{
    return eppv_SwapBytes_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapBytes_cs32, (const epp_cs32* pSrc, epp_cs32* pDst, epp_size length))
{
    return eppv_SwapBytes_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapBytes_cs64, (const epp_cs64* pSrc, epp_cs64* pDst, epp_size length))
{
    return eppv_SwapBytes_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapBytes_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
{
    return eppv_SwapBytes_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapBytes_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))
{
    return eppv_SwapBytes_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapBytes_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length))
{
    return eppv_SwapBytes_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length * 2);
}


/*
//  Name:        eppv_Flip
//  Purpose:     Inverts the vector element's order, pDst[i] = pSrc[length - 1 - i]
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_Flip_u16, (const epp_u16* pSrc, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Flip_u32, (const epp_u32* pSrc, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Flip_u64, (const epp_u64* pSrc, epp_u64* pDst, epp_size length))
EPPAPI(epp_Status, eppv_Flip_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length))
EPPINL(epp_Status, eppv_Flip_s16, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_Flip_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_s32, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_Flip_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_s64, (const epp_s64* pSrc, epp_s64* pDst, epp_size length))
{
    return eppv_Flip_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
{
    return eppv_Flip_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
{
    return eppv_Flip_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))
{
    return eppv_Flip_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_cs16, (const epp_cs16* pSrc, epp_cs16* pDst, epp_size length))
{
    return eppv_Flip_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_cs32, (const epp_cs32* pSrc, epp_cs32* pDst, epp_size length))
{
    return eppv_Flip_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_cs64, (const epp_cs64* pSrc, epp_cs64* pDst, epp_size length))
{
    return eppv_Flip_cf64((const epp_cf64*)pSrc, (epp_cf64*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
{
    return eppv_Flip_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_Flip_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))
{
    return eppv_Flip_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}


/*
//  Name:        eppv_RealToComplex
//  Purpose:     Creates complex vector from real and imaginary vectors
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrcRe     Pointer to the vector with real part of complex elements. If pDstRe == NULL then 0 is used as real part of elements.
//    pSrcIm     Pointer to the vector with imaginary part of complex elements. If pDstIm == NULL then 0 is used as imaginary part of elements.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_RealToComplex_f16cf16, (const epp_f16* pSrcRe, const epp_f16* pSrcIm, epp_cf16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_RealToComplex_f32cf32, (const epp_f32* pSrcRe, const epp_f32* pSrcIm, epp_cf32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_RealToComplex_f64cf64, (const epp_f64* pSrcRe, const epp_f64* pSrcIm, epp_cf64* pDst, epp_size length))
EPPINL(epp_Status, eppv_RealToComplex_s16cs16, (const epp_s16* pSrcRe, const epp_s16* pSrcIm, epp_cs16* pDst, epp_size length))
{
    return eppv_RealToComplex_f16cf16((const epp_f16*)pSrcRe, (const epp_f16*)pSrcIm, (epp_cf16*)pDst, length);
}
EPPINL(epp_Status, eppv_RealToComplex_s32cs32, (const epp_s32* pSrcRe, const epp_s32* pSrcIm, epp_cs32* pDst, epp_size length))
{
    return eppv_RealToComplex_f32cf32((const epp_f32*)pSrcRe, (const epp_f32*)pSrcIm, (epp_cf32*)pDst, length);
}
EPPINL(epp_Status, eppv_RealToComplex_s64cs64, (const epp_s64* pSrcRe, const epp_s64* pSrcIm, epp_cs64* pDst, epp_size length))
{
    return eppv_RealToComplex_f64cf64((const epp_f64*)pSrcRe, (const epp_f64*)pSrcIm, (epp_cf64*)pDst, length);
}


/*
//  Name:        eppv_ComplexToReal
//  Purpose:     Creates vectors with real and imaginary parts of complex vector
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       Pointer to the complex vector.
//    pDstRe     Pointer to the destination vector with real part of complex elements. If pDstRe == NULL then real part is skipped.
//    pDstIm     Pointer to the destination vector with imaginary part of complex elements. If pDstIm == NULL then imaginary part is skipped.
//    length     The length of vectors.
//  Notes:
//    If pDstRe is NULL
*/
EPPAPI(epp_Status, eppv_ComplexToReal_cf16f16, (const epp_cf16* pSrc, epp_f16* pDstRe, epp_f16* pDstIm, epp_size length))
EPPAPI(epp_Status, eppv_ComplexToReal_cf32f32, (const epp_cf32* pSrc, epp_f32* pDstRe, epp_f32* pDstIm, epp_size length))
EPPAPI(epp_Status, eppv_ComplexToReal_cf64f64, (const epp_cf64* pSrc, epp_f64* pDstRe, epp_f64* pDstIm, epp_size length))
EPPINL(epp_Status, eppv_ComplexToReal_cs16s16, (const epp_cs16* pSrc, epp_s16* pDstRe, epp_s16* pDstIm, epp_size length))
{
    return eppv_ComplexToReal_cf16f16((const epp_cf16*)pSrc, (epp_f16*)pDstRe, (epp_f16*)pDstIm, length);
}
EPPINL(epp_Status, eppv_ComplexToReal_cs32s32, (const epp_cs32* pSrc, const epp_s32* pDstRe, epp_s32* pDstIm, epp_size length))
{
    return eppv_ComplexToReal_cf32f32((const epp_cf32*)pSrc, (epp_f32*)pDstRe, (epp_f32*)pDstIm, length);
}
EPPINL(epp_Status, eppv_ComplexToReal_cs64s64, (const epp_cs64* pSrc, const epp_s64* pDstRe, epp_s64* pDstIm, epp_size length))
{
    return eppv_ComplexToReal_cf64f64((const epp_cf64*)pSrc, (epp_f64*)pDstRe, (epp_f64*)pDstIm, length);
}


/*
//  Name:        eppv_SwapEvenOdd
//  Purpose:     Swaps even and odd vector elements (d[0]=s[1], d[1]=s[0], d[2]=s[3], d[3]=s[2], ...)
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//    epp_StsBadParamsErr  length is odd
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, eppv_SwapEvenOdd_u16, (const epp_u16* pSrc, epp_u16* pDst, epp_size length))
EPPAPI(epp_Status, eppv_SwapEvenOdd_u32, (const epp_u32* pSrc, epp_u32* pDst, epp_size length))
EPPAPI(epp_Status, eppv_SwapEvenOdd_u64, (const epp_u64* pSrc, epp_u64* pDst, epp_size length))
EPPINL(epp_Status, eppv_SwapEvenOdd_s16, (const epp_s16* pSrc, epp_s16* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapEvenOdd_s32, (const epp_s32* pSrc, epp_s32* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapEvenOdd_s64, (const epp_s64* pSrc, epp_s64* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapEvenOdd_f16, (const epp_f16* pSrc, epp_f16* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapEvenOdd_f32, (const epp_f32* pSrc, epp_f32* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length);
}
EPPINL(epp_Status, eppv_SwapEvenOdd_f64, (const epp_f64* pSrc, epp_f64* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length);
}


/*
//  Name:        eppv_SwapReIm
//  Purpose:     Swaps Re and Im parts of vector elements (d[0].re=s[0].im, d[0].im=s[0].re, ...)
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse vector data pointer.
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPINL(epp_Status, eppv_SwapReIm_cs16, (const epp_cs16* pSrc, epp_cs16* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapReIm_cs32, (const epp_cs32* pSrc, epp_cs32* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapReIm_cs64, (const epp_cs64* pSrc, epp_cs64* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapReIm_cf16, (const epp_cf16* pSrc, epp_cf16* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u16((const epp_u16*)pSrc, (epp_u16*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapReIm_cf32, (const epp_cf32* pSrc, epp_cf32* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u32((const epp_u32*)pSrc, (epp_u32*)pDst, length * 2);
}
EPPINL(epp_Status, eppv_SwapReIm_cf64, (const epp_cf64* pSrc, epp_cf64* pDst, epp_size length))
{
    return eppv_SwapEvenOdd_u64((const epp_u64*)pSrc, (epp_u64*)pDst, length * 2);
}

#ifdef __cplusplus
}
#endif
